-- main.lua for bezier.love

function love.load()

	gr = love.graphics
	gr.setCaption( "Bezier vs Cubic Spline Curve Fitting" )
	x = { 100, 250, 550, 700 }
	y = { 500, 250, 250, 500 }
	gr.setBackgroundColor( 55, 55,195, 255 )
	gr.setFont( gr.newFont( 18 ) )
	gr.setLineWidth(2)
	end

function love.draw()

	gr.setColor(195, 195, 0, 255)
	for i = 1, #x - 1 do gr.line( x[i], y[i], x[i+1], y[i+1] ) end

	--gr.setColor( 25, 0, 50, 255 )
	for i=1,#x do circle( x[i], y[i], 8 ) end

	gr.setColor( 255, 0, 0, 255 )
	data = Bezier( x, y, 0.01)
	for i = 1, #data - 1 do
		gr.line( data[i][1], data[i][2], data[i+1][1], data[i+1][2] )
		end
	gr.print("Bezier Curve", 540, 550 )
	gr.setColor( 0, 0, 255, 255 )
	spline = cubicSpline( x, y )
	for i = 1, #spline - 1 do
		gr.line( spline[i][1], spline[i][2], spline[i+1][1],spline[i+1][2] )
		end
	gr.print("Cubuc Spline Curve", 540,570 )
	gr.setColor(255, 255, 0, 255)
	gr.print("FPS: " .. love.timer.getFPS(), 10, 10)
	text = "X1 Y1: "..x[1].." - "..y[1].."\nX2 Y2: "..x[2].." - "..y[2].."\nX3 Y3: "..x[3].." - "..y[3].."\nX4 Y4: "..x[4].." - "..y[4]
	gr.print(text,10,510)

	end

function love.mousepressed(mx, my, button)

	if button == "l" then
		d = {}
		for i = 1, 4 do d[i] = math.sqrt( (x[i]-mx)^2 + (y[i]-my)^2) end
		min_val = math.min( d[1], math.min( d[2], math.min( d[3], d[4] ) ) )
		for i = 1,#d do if d[i] == min_val then x[i] = mx y[i] = my end end
		end

	end

function love.keypressed( key )

	if key == "escape" then love.event.quit() end

	end

function circle( cx, cy, r ) gr.circle( 'line', cx, cy, r, r ) end

function Bezier(px,py,steps)

	local bezier = {} index = 0      -- bezier is a table of xy pairs
	local cx = 3 * ( px[ 2 ] - px[ 1 ] )
	local cy = 3 * ( py[ 2 ] - py[ 1 ] )
	local bx = 3 * ( px[ 3 ] - px[ 2 ]) - cx
	local by = 3 * ( py[ 3 ] - py[ 2 ]) - cy
	local ax = px[ 4 ] - px[ 1 ] - cx - bx
	local ay = py[ 4 ] - py[ 1 ] - cy - by
	index = index + 1
	bezier[index]={px[1],py[1]}
	for t = 0, 1, steps do  -- steps typically equals .01
		local X = ax * t^3 + bx * t^2 + cx * t + px[ 1 ]
		local Y = ay * t^3 + by * t^2 + cy * t + py[ 1 ]
		index = index + 1
		bezier[ index ] = { X, Y }
		end
	return bezier

	end

function cubicSpline(sx,sy)
	-- Original Basic version by Charles O'Neill 2002
	-- 'x' values don't have to be uniformly spaced
	local A = {} B = {} C = {} D = {} h = {} S = {}
	for i = 1, #sx - 1 do			-- width of each 'x' interval
		h[i] = sx[i+1] - sx[i]
		end
	for i = 2, #sx - 1 do
		j = i - 1
		A[j] = h[i]
		B[j] = h[i-1]
		D[j] = 2 * ( h[i-1] + h[i] )
		end
	for i = 2, #sx - 1 do			-- Matrix C (constants)
		j = i - 1
		C[j] = 6 * (( sy[i+1] - sy[i])/h[i] - (sy[i] - sy[i-1]) / h[i-1])
		end
	local ntdma = #sx - 2			-- tdma = Tre-Diagonal-Matrix-Algorithm
	for i = 2, ntdma do
		R = B[i] / D[i-1]
		C[i] = C[i] - R * C[i-1]
		D[i] = D[i] - R * A[i-1]
		end
	C[ntdma] = C[ntdma] / D[ntdma]
	for i = ntdma - 1, 1, -1 do
		C[i] = (C[i] - A[i] * C[i+1]) / D[i]
		end
	for i	= 2, #sx - 1 do			-- changes from zero based index to 1
		j	= i - 1
		S[i] = C[j]					-- S is the second derivative
		end
	S[1]	= 0						-- set up end coordinates
	S[#sx]	= 0
	for i	= 1, #sx - 1 do          -- calculate cubic coordinates
		A[i] = (S[i+1] - S[i]) / (6 * h[i] )
		B[i] = S[i] / 2
		C[i] = (sy[i+1] - sy[i]) / h[i] - (2 * h[i] * S[i] + h[i] * S[i+1]) / 6
		D[i] = sy[i]
		end
	spline, count = {}, 1
	nstep = 10						-- number of points create btw initial points
	for i = 1, #sx - 1 do			-- spline is a table of fitted x,y values
		for j = 1, nstep do
			local x = sx[i] + ( h[i] / nstep ) * ( j - 1 )
			local dif = x - sx[i]
			local y = A[i] * dif ^ 3 + B[i] * dif ^ 2 + C[i] * dif + D[i]
			spline[count] = { math.floor(x), math.floor(y) }
			count = count + 1
			end
		end
	spline[ #spline + 1 ] = { sx[ #sx ], sy[ #sx ] } -- adds last point to table
	return spline
	end

